use std::fmt::{Display, Formatter};
use bright::Bright;

peg::parser! {
    grammar input_parser() for str {
        rule whitespace() = quiet!{[' ' | '\n' | '\t']+}

        rule number() -> usize
          = n:$(['0'..='9']+) {? n.parse().or(Err("u32"))}

        rule starting_line() -> Vec<usize> = number() ** ","

        rule num_whitespace() -> usize = a:number() whitespace()? { a }

        rule bingo_board() -> BingoBoard
          = a:num_whitespace()*<25> { a.into() }

        rule bingo_boards() -> Vec<BingoBoard> = bingo_board()*

        pub(crate) rule bingo_game() -> BingoGame
        = a:starting_line() "\n"* b:bingo_boards()  { BingoGame { numbers: a, boards: b } }
    }
}

#[derive(Copy, Clone)]
enum Seen {
    NotSeen,
    Seen,
}

#[derive(Copy, Clone)]
struct BingoTile {
    num: usize,
    seen: Seen
}

impl Display for BingoTile {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        let a = match self.seen {
            Seen::NotSeen => { Bright::new(format!("{}", self.num)).red().to_string() }
            Seen::Seen => { Bright::new(format!("{}", self.num)).green().to_string() }
        };
        write!(f, "{}", a)
    }
}

impl From<usize> for BingoTile {
    fn from(num: usize) -> Self {
        BingoTile {
            num,
            seen: Seen::NotSeen
        }
    }
}

impl TryFrom<&str> for BingoTile {
    type Error = ();

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        Ok(value.parse::<usize>().unwrap().into())
    }
}

impl BingoTile {
    fn see(&self) -> Self{
        BingoTile {
            num: self.num,
            seen: Seen::Seen
        }
    }

    fn see_num(&self, num: usize) -> Self {
        if self.num == num {
            Self {
                num: self.num,
                seen: Seen::Seen
            }
        } else {
            self.clone()
        }
    }

    fn see_mut(&mut self) -> () {
        self.seen = Seen::Seen
    }

    fn see_num_mut(&mut self, num: usize) -> () {
        if self.num == num {
            self.seen = Seen::Seen
        }
    }
}

struct BingoBoard {
    tiles: Vec<BingoTile>
}

impl BingoBoard {
    fn new() -> Self {
        BingoBoard {
            tiles: vec![]
        }
    }

    fn see_num(&self, num: usize) -> Self {
        Self { tiles: self.tiles.iter().map(|tile| tile.see_num(num)).collect() }
    }

    fn see_num_mut(&mut self, num: usize) -> () {
        self.tiles.iter_mut().for_each(|mut tile| tile.see_num_mut(num))
    }
}

impl Display for BingoBoard {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        let longest = self.tiles.iter().max_by(|&x, &y| x.to_string().len().cmp(&y.to_string().len()) ).unwrap().num.to_string().len();
        let mut out_str = String::new();
        self.tiles.iter().enumerate().for_each(|(idx, x)| {if idx % 5 == 0 { out_str.push_str("\n") }; out_str.push_str(format!("{:width$} ", x, width = longest).as_str())});
        write!(f, "{}", out_str)
    }
}

impl From<Vec<usize>> for BingoBoard {
    fn from(value: Vec<usize>) -> Self {
        BingoBoard {
            tiles: value.iter().map(|&num| num.into()).collect()
        }
    }
}

struct BingoGame {
    boards: Vec<BingoBoard>,
    numbers: Vec<usize>
}


impl BingoGame {
    fn see(&self, num: usize) -> Self {
        let boards = self.boards.iter().map(|board| board.see_num(num)).collect();
        Self {
            numbers: self.numbers.clone(),
            boards
        }
    }
}

impl Display for BingoGame {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        let mut out_str = String::new();
        self.numbers.iter().for_each(| x| out_str.push_str(format!("{},", x.to_string()).as_str()));
        out_str.push_str("\n");
        self.boards.iter().for_each(|board| out_str.push_str(format!("{}\n\n", board.to_string()).as_str()));
        write!(f, "{}", out_str)
    }
}

fn parse_input(input: &str) -> BingoGame {
    input_parser::bingo_game(input).unwrap()
}

/// The submarine has a bingo subsystem to help passengers (currently, you and the giant squid) pass the time. It automatically generates a random order in which to draw numbers and a random set of boards (your puzzle input). For example:
///
/// `7,4,9,5,11,17,23,2,0,14,21,24,10,16,13,6,15,25,12,22,18,20,8,19,3,26,1`
///
/// ```text
/// 22 13 17 11  0
///  8  2 23  4 24
/// 21  9 14 16  7
///  6 10  3 18  5
///  1 12 20 15 19
///
///  3 15  0  2 22
///  9 18 13 17  5
/// 19  8  7 25 23
/// 20 11 10 24  4
/// 14 21 16 12  6
///
/// 14 21 17 24  4
/// 10 16 15  9 19
/// 18  8 23 26 20
/// 22 11 13  6  5
///  2  0 12  3  7
/// ```
///
/// After the first five numbers are drawn (7, 4, 9, 5, and 11), there are no winners, but the boards are marked as follows (shown here adjacent to each other to save space):
///
/// ```text
/// 22 13 17 11  0         3 15  0  2 22        14 21 17 24  4
///  8  2 23  4 24         9 18 13 17  5        10 16 15  9 19
/// 21  9 14 16  7        19  8  7 25 23        18  8 23 26 20
///  6 10  3 18  5        20 11 10 24  4        22 11 13  6  5
///  1 12 20 15 19        14 21 16 12  6         2  0 12  3  7
/// ```
///
/// After the next six numbers are drawn (17, 23, 2, 0, 14, and 21), there are still no winners:
///
/// ```text
/// 22 13 17 11  0         3 15  0  2 22        14 21 17 24  4
///  8  2 23  4 24         9 18 13 17  5        10 16 15  9 19
/// 21  9 14 16  7        19  8  7 25 23        18  8 23 26 20
///  6 10  3 18  5        20 11 10 24  4        22 11 13  6  5
///  1 12 20 15 19        14 21 16 12  6         2  0 12  3  7
/// ```
///
/// Finally, 24 is drawn:
///
/// ```text
/// 22 13 17 11  0         3 15  0  2 22        14 21 17 24  4
///  8  2 23  4 24         9 18 13 17  5        10 16 15  9 19
/// 21  9 14 16  7        19  8  7 25 23        18  8 23 26 20
///  6 10  3 18  5        20 11 10 24  4        22 11 13  6  5
///  1 12 20 15 19        14 21 16 12  6         2  0 12  3  7
/// ```
///
/// At this point, the third board wins because it has at least one complete row or column of marked numbers (in this case, the entire top row is marked: 14 21 17 24 4).
///
/// The score of the winning board can now be calculated. Start by finding the sum of all unmarked numbers on that board; in this case, the sum is 188. Then, multiply that sum by the number that was just called when the board won, 24, to get the final score, 188 * 24 = 4512.
pub fn part1(input: &str) -> usize {
    let a = parse_input(input);
    println!("{}", a);

    for called_number in a.numbers {
    }
    0
}


#[cfg(test)]
mod tests {
    #[test]
    fn it_works() {
        let result = 2 + 2;
        assert_eq!(result, 4);
    }
}
